import pygame
from constants import *
from utils import *

# TODO: create a utils.py class for cleaner setup


#the Heads Up Display blitted onto the screen in the game loop
class Hud():

    def __init__(self):

        #get images
        self.bone = pygame.image.load("assets/hud/goldenbone.png")
        self.coin = pygame.image.load("assets/hud/coin_hud.png")
        self.p2_lives_icon = pygame.image.load("assets/hud/indiana_bones_lives_hud.png")
        self.p1_lives_icon = pygame.image.load("assets/hud/player1_lives_hud.png")
        self.lives_background = pygame.image.load("assets/hud/background.png")
        self.lives_background_active = pygame.transform.scale(self.lives_background, (106.6, 39))
        self.col_background = pygame.image.load("assets/hud/background_3.png")

        #when game is run in singleplayer, the HUD displays the active character bigger
        self.p2_lives_icon_active = pygame.transform.scale(self.p2_lives_icon, (33.8, 27.3))
        self.p1_lives_icon_active = pygame.transform.scale(self.p1_lives_icon, (20.8, 27.5))

        #the max amount of coins in each level
        self.max_coin_count = '30'

        #ge the middle of widht and height of screen
        self.mid_w = WIDTH / 2
        self.mid_h = HEIGHT / 2

        #the digits of the stopwatch get rendered at initialization to enhance performance
        #once the time is updated, we simply display the new digit
        self.rendered_digits = {
            str(i): create_text_surf(str(i), 13, (237, 189, 78))
            for i in range(10)
        }
        self.rendered_digits[":"] = create_text_surf(":", 13, (237, 189, 78))
        self.rendered_digits["/"] = create_text_surf("/", 13, (237, 189, 78))


    def hud_init(self, p1_lives, p2_lives, coin_counter, bone_counter):
        #initialize the text elements of the hud (count of lives, coins, bone and time)
        #called in the game_loop when the game is initialized
        self.p1_lives_surf = create_text_surf(str(p1_lives), 15, (237, 189, 78))
        self.p2_lives_surf = create_text_surf(str(p2_lives), 15, (237, 189, 78))
        self.p1_lives_surf_active = create_text_surf(str(p1_lives), 20, (237, 189, 78))
        self.p2_lives_surf_active = create_text_surf(str(p2_lives), 20, (237, 189, 78))

        self.coin_counter_surf = pygame.Surface((60,20), pygame.SRCALPHA)
        self.coin_counter_surf.set_colorkey((0,0,0))
        self.update_coin(0)

        self.bone_counter_surf = pygame.Surface((60,20), pygame.SRCALPHA)
        self.bone_counter_surf.set_colorkey((0,0,0))
        self.update_bones(0)

        self.stopwatch_surf = pygame.Surface((60,20), pygame.SRCALPHA)
        self.stopwatch_surf.set_colorkey((0,0,0))
        self.update_stopwatch("00:00")

    #the following methods are called from the game loop when a count changes
    def update_p1_lives(self, p1_lives):
        self.p1_lives_surf = create_text_surf(str(p1_lives), 15, (237, 189, 78))
        self.p1_lives_surf_active = create_text_surf(str(p1_lives), 20, (237, 189, 78))

    def update_p2_lives(self, p2_lives):
        self.p2_lives_surf = create_text_surf(str(p2_lives), 15, (237, 189, 78))
        self.p2_lives_surf_active = create_text_surf(str(p2_lives), 20, (237, 189, 78))

    def update_coin(self, coin_counter):
        cur_x = 0
        self.coin_counter_surf.fill((0, 0, 0))
        for char in f"{coin_counter}/{self.max_coin_count}":
            digit_surf = self.rendered_digits[char]
            self.coin_counter_surf.blit(digit_surf, dest=(cur_x, 0))
            cur_x += digit_surf.get_width()

    def update_bones(self, bone_counter):
        cur_x = 0
        self.bone_counter_surf.fill((0, 0, 0))
        for char in f"{bone_counter}":
            digit_surf = self.rendered_digits[char]
            self.bone_counter_surf.blit(digit_surf, dest=(cur_x, 0))
            cur_x += digit_surf.get_width()

    #when the time changes, we update the stopwatch per digit and display pre rendered digits (initialized in HUD init)
    def update_stopwatch(self, stopwatch):
        cur_x = 0
        self.stopwatch_surf.fill((0, 0, 0))
        for char in stopwatch:
            digit_surf = self.rendered_digits[char]
            self.stopwatch_surf.blit(digit_surf, dest=(cur_x, 0))
            cur_x += digit_surf.get_width()


    #the blitting operation that is colled in game_loop continously
    def display_hud(self, menu_state, active_p1, active_p2):

        if menu_state == "Singleplayer":
            if active_p1:
                displaysurface.blit(self.lives_background_active, (5, 5))
                displaysurface.blit(self.p1_lives_icon_active, (30, 12))
                displaysurface.blit(self.p1_lives_surf_active, dest=(65, 12))

            else:
                displaysurface.blit(self.lives_background, (5, 5))
                displaysurface.blit(self.p1_lives_icon, (25, 10.5))
                displaysurface.blit(self.p1_lives_surf, dest=(55, 10))

            if active_p2:
                displaysurface.blit(self.lives_background_active,
                                    (WIDTH - (self.lives_background_active.get_width() + 5), 5))
                displaysurface.blit(self.p2_lives_icon_active, (WIDTH - 90, 12))
                displaysurface.blit(self.p2_lives_surf_active, dest=(WIDTH - 45, 11))

            else:
                displaysurface.blit(self.lives_background, (WIDTH - (self.lives_background.get_width() + 5), 5))
                displaysurface.blit(self.p2_lives_icon, (WIDTH - 70, 10))
                displaysurface.blit(self.p2_lives_surf, dest=(WIDTH - 35, 10))

        if menu_state == "Multiplayer":
            displaysurface.blit(self.lives_background, (5, 5))
            displaysurface.blit(self.p1_lives_icon, (25, 10.5))
            displaysurface.blit(self.p1_lives_surf, dest=(55, 10))
            displaysurface.blit(self.lives_background, (WIDTH - (self.lives_background.get_width() + 5), 5))
            displaysurface.blit(self.p2_lives_icon, (WIDTH - 70, 10))
            displaysurface.blit(self.p2_lives_surf, dest=(WIDTH - 35, 10))

        displaysurface.blit(self.col_background, ((WIDTH / 2) - (self.col_background.get_width() / 2), 5))

        displaysurface.blit(self.stopwatch_surf, dest=(280, 12))

        displaysurface.blit(self.coin, (353, 15))
        displaysurface.blit(self.coin_counter_surf, dest=(368, 12))

        displaysurface.blit(self.bone, (450, 15))
        displaysurface.blit(self.bone_counter_surf, dest=(475, 12))
