import sys
import time

import json

import pygame
from constants import *
from logo import Logo
from indiana_bones import IndianaBones
from karl import Karl
from levels import Level
from pyvidplayer import Video
from utils import *

from pygame import mixer, QUIT


#the entry object for an entry in the highscore database
class Entry(object):
    def __init__(self, name, time, coins, bone):
        self.name = name
        self.time = time
        self.coins = coins
        self.bone = bone

#general menu class, which every menu is a type of from
class Menu():

    def __init__(self, game):
        #set general variables every menu needs
        self.WHITE = (255, 255, 255)
        self.BLACK = (0, 0, 0)
        self.YELLOW = (237, 189, 78)

        self.game = game
        self.mid_w = WIDTH / 2
        self.mid_h = HEIGHT / 2

        self.run_menu = False
        self.UP_KEY, self.DOWN_KEY, self.START_KEY, self.BACK_KEY, self.HIGHSCORE_KEY = False, False, False, False, False

        self.cursor_surf = create_text_surf('x', 15, (237, 189, 78))

        self.cursor_offset_x = -35

        self.debouncing_timer = 0

        self.level_select_sound = pygame.mixer.Sound("assets/sounds/level_selected.wav")
        self.select_sound = pygame.mixer.Sound("assets/sounds/select.wav")
        self.enter_sound = pygame.mixer.Sound("assets/sounds/validate.wav")

    #update the screen and reset key detection
    def blit_screen(self):
        fps = FramePerSec.get_fps()
        #print(fps)
        pygame.display.flip()
        FramePerSec.tick(FPS)
        self.reset_keys()

    #reset key variables after every loop to detect joystick and keyboard controls
    def reset_keys(self):
        self.UP_KEY, self.DOWN_KEY, self.START_KEY, self.BACK_KEY, self.HIGHSCORE_KEY = False, False, False, False, False


#the main menu of the game which lets you choose to play in either single- or multiplayer, roll the credits and quit the game
class MainMenu(Menu):
    def __init__(self, game):
        Menu.__init__(self, game)

        # selected item of cursor
        self.state = "Singleplayer"

        # position of items of menu
        self.singleplayer_x, self.singleplayer_y = 150, self.mid_h -5
        self.multiplayer_x, self.multiplayer_y = 150, self.mid_h + 25
        self.credits_x, self.credits_y = 150, self.mid_h + 55
        self.quit_x, self.quit_y = 150, self.mid_h + 85

        # put cursor on first item (uses the cursor offset of menu class)
        self.cursor_pos = (self.singleplayer_x + self.cursor_offset_x, self.singleplayer_y)

        # display needed sprites
        self.logo = Logo()
        self.IB = IndianaBones()
        self.karl = Karl()

        #play music
        mixer.music.load('assets/music/adventure-movie-amp-animation-soundtrack-1230.mp3')
        mixer.music.set_volume(1.0)
        mixer.music.play(loops=-1)

        #variable to stay in/leave display_menu loop from menu class
        self.run_menu = True

    #method to display the current menu
    #method is called from main.py once the game sets it's game.inMenu variable
    def display_menu(self):
        #do some basic pre operation before displaying
        self.run_menu = True

        self.game.fire_channel.stop()

        #move the logo to the fitting the position
        self.logo.move(WIDTH / 2, 120)

        bg1 = pygame.image.load("assets/background/plx-1.png")
        bg1 = pygame.transform.scale(bg1, (WIDTH + 300, HEIGHT))

        bg2 = pygame.image.load("assets/background/plx-2.png")
        bg2 = pygame.transform.scale(bg2, (WIDTH + 300, HEIGHT))

        bg3 = pygame.image.load("assets/background/plx-3.png")
        bg3 = pygame.transform.scale(bg3, (WIDTH + 300, HEIGHT))

        bg4 = pygame.image.load("assets/background/plx-4.png")
        bg4 = pygame.transform.scale(bg4, (WIDTH + 300, HEIGHT))

        bg5 = pygame.image.load("assets/background/plx-5.png")
        bg5 = pygame.transform.scale(bg5, (WIDTH + 300, HEIGHT))

        ground = pygame.image.load("assets/background/ground.png")
        ground = pygame.transform.scale(ground, (320, 64))

        low_opacity = pygame.Surface((WIDTH, 127))  # the size of your rect
        low_opacity.set_alpha(150)  # alpha level
        low_opacity.fill((0, 0, 0))  # this fills the entire surface

        singleplayer_surf = create_text_surf("Singleplayer", 20, (237, 189, 78))
        multiplayer_surf = create_text_surf("Multiplayer", 20, (237, 189, 78))
        credits_surf = create_text_surf("Credits", 20, (237, 189, 78))
        quit_surf = create_text_surf("Quit Game", 20, (237, 189, 78))

        background_surface = pygame.Surface((WIDTH, HEIGHT))
        background_surface.blit(bg1, (0, 0))
        background_surface.blit(bg2, (0, 0))
        background_surface.blit(bg3, (0, 0))
        background_surface.blit(bg4, (0, 0))
        background_surface.blit(bg5, (0, 0))
        background_surface.blit(low_opacity, dest=(0, self.mid_h - 10))
        background_surface.blit(singleplayer_surf, dest=(self.singleplayer_x, self.singleplayer_y))
        background_surface.blit(multiplayer_surf, dest=(self.multiplayer_x, self.multiplayer_y))
        background_surface.blit(credits_surf, dest=(self.credits_x, self.credits_y))
        background_surface.blit(quit_surf, dest=(self.quit_x, self.quit_y))
        background_surface.blit(ground, (0,HEIGHT-48))
        background_surface.blit(ground, (320, HEIGHT - 48))
        background_surface.blit(ground, (640, HEIGHT - 48))

        #display the menu
        while self.run_menu:
            #check for button presses and move the cursor if needed
            self.check_events()
            self.move_cursor()

            displaysurface.blit(background_surface, dest=(0,0))

            #make the sprites move
            self.logo.update()
            self.IB.update()
            self.karl.update()

            displaysurface.blit(self.logo.surf, self.logo.rect)

            #show Indiana Bones and Karl depending on state of menu
            if self.state == 'Singleplayer':
                self.IB.move(730, 280)
                displaysurface.blit(self.IB.surf, self.IB.rect)

            if self.state == 'Multiplayer':
                self.IB.move(650, 280)
                displaysurface.blit(self.IB.surf, self.IB.rect)
                self.karl.move(800, 250)
                displaysurface.blit(self.karl.surf, self.karl.rect)


            displaysurface.blit(self.cursor_surf, dest=self.cursor_pos)
            self.blit_screen()

    # check for events on the controls and set universal key variables used by keyboard and joystick
    def check_events(self):
        self.debouncing_timer += 1
        #print(self.debouncing_timer)
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.game.running, self.game.playing = False, False
                self.run_menu = False
            if INPUT_METHOD == "KEYBOARD":
                if event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_RETURN:
                        self.START_KEY = True
                    if event.key == pygame.K_BACKSPACE:
                        self.BACK_KEY = True
                    if event.key == pygame.K_DOWN:
                        self.DOWN_KEY = True
                    if event.key == pygame.K_UP:
                        self.UP_KEY = True
                    if event.key == self.game.COIN_KEY:
                        self.game.wallet.increase_balance()

            elif INPUT_METHOD == "JOYSTICK":
                if self.debouncing_timer > 15:
                    if event.type == pygame.JOYBUTTONDOWN:
                        if event.button == 0:
                            self.START_KEY = True
                            self.debouncing_timer = 0
                        if event.button == 4:
                            self.BACK_KEY = True
                            self.debouncing_timer = 0
                    if event.type == pygame.JOYAXISMOTION:
                        if event.axis == 1 and event.value > .95:
                            self.DOWN_KEY = True
                            self.debouncing_timer = 0
                        if event.axis == 1 and event.value < -.95:
                            self.UP_KEY = True
                            self.debouncing_timer = 0

    #move cursor depending on button press and change menus on select
    def move_cursor(self):
        if self.START_KEY:
            if self.state == 'Singleplayer' or self.state == "Multiplayer":
                self.enter_sound.play()
                self.run_menu = False
                self.game.curr_menu = self.game.level_select

            elif self.state == 'Quit Game':
                self.game.running, self.game.playing = False, False
                self.run_menu = False
            elif self.state == "Credits":
                # We use the pyvidplayer library to run our video
                mixer.music.stop()
                vid = Video("./assets/credits/creditswithlogo.mp4")
                vid.set_size((WIDTH, HEIGHT))
                vid.set_volume(.2)
                # this is a loop that runs while the video is playing. We simply blit the current video frame
                # onto the display surface
                while vid.active:
                    for event in pygame.event.get():
                        if event.type == QUIT:
                            pygame.quit()
                            sys.exit()
                    vid.draw(displaysurface, (0, 0))
                    FramePerSec.tick(60)
                    pygame.display.flip()
                vid.close()
                mixer.music.set_volume(1.0)
                mixer.music.play(loops=-1)
        if self.DOWN_KEY:
            self.select_sound.play()
            if self.state == 'Singleplayer':
                self.cursor_pos = (self.multiplayer_x + self.cursor_offset_x, self.multiplayer_y)
                self.state = 'Multiplayer'
            elif self.state == 'Multiplayer':
                self.cursor_pos = (self.credits_x + self.cursor_offset_x, self.credits_y)
                self.state = 'Credits'
            elif self.state == 'Credits':
                self.cursor_pos = (self.quit_x + self.cursor_offset_x, self.quit_y)
                self.state = 'Quit Game'
            elif self.state == 'Quit Game':
                self.cursor_pos = (self.singleplayer_x + self.cursor_offset_x, self.singleplayer_y)
                self.state = 'Singleplayer'
        elif self.UP_KEY:
            self.select_sound.play()
            if self.state == 'Singleplayer':
                self.cursor_pos = (self.quit_x + self.cursor_offset_x, self.quit_y)
                self.state = 'Quit Game'
            elif self.state == 'Multiplayer':
                self.cursor_pos = (self.singleplayer_x + self.cursor_offset_x, self.singleplayer_y)
                self.state = 'Singleplayer'
            elif self.state == 'Credits':
                self.cursor_pos = (self.multiplayer_x + self.cursor_offset_x, self.multiplayer_y)
                self.state = 'Multiplayer'
            elif self.state == 'Quit Game':
                self.cursor_pos = (self.credits_x + self.cursor_offset_x, self.credits_y)
                self.state = 'Credits'


#the level selection menu, allows to select a level and display highscores
class LevelSelection(Menu):
    def __init__(self, game):
        Menu.__init__(self, game)

        # selected item of cursor
        self.state = "Level 1"
        # index of selected level in levels list
        self.level_index = 0

        #use custom cursor offset, different from Menu class
        self.cursor_offset_x = -25

        #variables needed to determine state of popup window
        #indicates if the pop up window is shown or not
        self.show_database = False
        #indiactes whether the cursor is allowed to move or not
        self.cursor_active = True
        #indicates whether a button press closes or opens the op up window containg the database
        self.counter_button_highscore = 0

        self.title_x, self.title_y = 58, 39
        self.back_x, self.back_y = 58, 350
        self.run_menu = False

        #INITIALIZE LEVEL DATA
        # indiv level tuples with .tmx and .json data
        level1_tuple = ("./assets/level_1/StartLevel.tmx", "./assets/level_1/door_data.json")
        level2_tuple = ("./assets/level_demo/level_demo.tmx", "assets/level_demo/door_data_json")
        level3_tuple = ("./assets/level_2/Level2.tmx", "assets/level_2/door_data_json")
        level4_tuple = ("./assets/level_3/Level3.tmx", "assets/level_3/door_data_json")
        level5_tuple = ("./assets/level_4/Level4.tmx", "assets/level_4/door_data_json")
        level6_tuple = ("./assets/level_5/Level5.tmx", "assets/level_5/door_data_json")
        level7_tuple = ("./assets/level_6/Level6.tmx", "assets/level_6/door_data_json")

        #indiv level thumbnails to see in level selection
        level1_thumb = pygame.image.load("assets/level_1/thumbnail_level1.png")
        level1_thumb = pygame.transform.scale(level1_thumb, (129, 175.5))
        level2_thumb = pygame.image.load("assets/level_demo/thumbnail_leveldemo.png")
        level2_thumb = pygame.transform.scale(level2_thumb, (129, 175.5))
        level3_thumb = pygame.image.load("assets/level_2/thumbnail_level2.png")
        level3_thumb = pygame.transform.scale(level3_thumb, (129, 175.5))
        level4_thumb = pygame.image.load("assets/level_3/thumbnail_level3.png")
        level4_thumb = pygame.transform.scale(level4_thumb, (129, 175.5))
        level5_thumb = pygame.image.load("assets/level_4/thumbnail_level4.png")
        level5_thumb = pygame.transform.scale(level5_thumb, (129, 175.5))
        level6_thumb = pygame.image.load("assets/level_5/thumbnail_level5.png")
        level6_thumb = pygame.transform.scale(level6_thumb, (129, 175.5))
        level7_thumb = pygame.image.load("assets/level_6/thumbnail_level6.png")
        level7_thumb = pygame.transform.scale(level7_thumb, (129, 175.5))

        #indiv level titles
        level1_title_surf = create_text_surf("1: Ladder Up", 20, (237, 189, 78))
        level2_title_surf = create_text_surf("2: Learning by Doing", 20, (237, 189, 78))
        level3_title_surf = create_text_surf("3: Spike Nightmare", 20, (237, 189, 78))
        level4_title_surf = create_text_surf("4: Is it hot in here?", 20, (237, 189, 78))
        level5_title_surf = create_text_surf("5: Is it worth it?", 20, (237, 189, 78))
        level6_title_surf = create_text_surf("6: Boxy McBox Face", 20, (237, 189, 78))
        level7_title_surf = create_text_surf("7: Bridge Building", 20, (237, 189, 78))

        #array objects of level data to use for navigation between levels
        self.levels = [level1_tuple, level2_tuple, level3_tuple, level4_tuple, level5_tuple, level6_tuple, level7_tuple]
        self.level_title_surfs = [level1_title_surf, level2_title_surf, level3_title_surf, level4_title_surf, level5_title_surf,
                                  level6_title_surf, level7_title_surf]
        self.level_title_coor = [(58, 100),(58, 130),(58, 160),(58, 190),(58, 220),(58, 250),(58, 280)]
        self.level_thumbnails = [level1_thumb, level2_thumb, level3_thumb, level4_thumb, level5_thumb, level6_thumb, level7_thumb]
        self.level_difficulties = ["Easy", "Easy", "Medium", "Medium", "Medium", "Hard", "Hard"]
        self.level_highscores = ['highscore_files/level1.json', 'highscore_files/level2.json', 'highscore_files/level3.json',
                                 'highscore_files/level4.json', 'highscore_files/level5.json', 'highscore_files/level6.json',
                                 'highscore_files/level7.json']

        #setup the titles for the pop up window of the highscores
        #we only display the top 10, get good!
        pop_up_title_lvl1 = create_text_surf("The Top 10 of Level 1", 17, (237, 189, 78))
        pop_up_title_lvl2 = create_text_surf("The Top 10 of Level 2", 17, (237, 189, 78))
        pop_up_title_lvl3 = create_text_surf("The Top 10 of Level 3", 17, (237, 189, 78))
        pop_up_title_lvl4 = create_text_surf("The Top 10 of Level 4", 17, (237, 189, 78))
        pop_up_title_lvl5 = create_text_surf("The Top 10 of Level 5", 17, (237, 189, 78))
        pop_up_title_lvl6 = create_text_surf("The Top 10 of Level 6", 17, (237, 189, 78))
        pop_up_title_lvl7 = create_text_surf("The Top 10 of Level 7", 17, (237, 189, 78))

        self.pop_up_titles = [pop_up_title_lvl1, pop_up_title_lvl2, pop_up_title_lvl3, pop_up_title_lvl4,
                              pop_up_title_lvl5, pop_up_title_lvl6, pop_up_title_lvl7]

        #used to convert the highscores of all levels from .json to text
        self.data_as_text = []

        #displayed if there are no highscores
        self.no_highscores = create_text_surf("No highscores yet", 17, (237, 189, 78))

        #the pop up window that will display the highscores
        self.pop_up_window = pygame.image.load("assets/hud/pop_up_window.png")
        self.pop_up_window = pygame.transform.scale(self.pop_up_window,
                                               (self.pop_up_window.get_width() * 1.4, self.pop_up_window.get_height() * 1.4))

        coin = pygame.image.load("assets/hud/coin_hud.png")
        bone = pygame.image.load("assets/hud/goldenbone.png")
        name = create_text_surf("Name", 15, (255, 255, 255))
        time = create_text_surf("Time", 15, (255, 255, 255))

        self.pop_up_window.blit(name, dest=(35, 55))
        self.pop_up_window.blit(bone, dest=(215, 62))
        self.pop_up_window.blit(coin, dest=(400, 62))
        self.pop_up_window.blit(time, dest=(575, 55))


        #the wallet is displayed so that players see the current amount of coins
        #red wallet state used to indicate whether or not the player can afford to start a level
        self.red_wallet = True
        #displayed depending on wallets amount of coins
        self.red_wallet_surf = create_text_surf("Your Wallet:", 15, (255, 0, 0))
        self.yellow_wallet_surf = create_text_surf("Your Wallet:", 15, (237, 189, 78))

        #played when people can not afford to play a level
        self.wallet_error_sound = pygame.mixer.Sound("assets/sounds/wallet_error.wav")

        # put cursor on first item
        self.cursor_pos = (self.level_title_coor[0][0] + self.cursor_offset_x, self.level_title_coor[0][1])


    #check for events on the controls and set universal key variables used by keyboard and joystick
    def check_events(self):
        self.debouncing_timer += 1
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.game.running, self.game.playing = False, False
                self.run_menu = False
            if INPUT_METHOD == "KEYBOARD":
                if event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_RETURN:
                        self.START_KEY = True
                    if event.key == pygame.K_BACKSPACE:
                        self.BACK_KEY = True
                    if event.key == pygame.K_DOWN:
                        self.DOWN_KEY = True
                    if event.key == pygame.K_UP:
                        self.UP_KEY = True
                    if event.key == self.game.COIN_KEY:
                        self.game.wallet.increase_balance()
                    if event.key == pygame.K_d:
                        self.HIGHSCORE_KEY = True
            elif INPUT_METHOD == "JOYSTICK":
                if self.debouncing_timer > 15:
                    if event.type == pygame.JOYBUTTONDOWN:
                        if event.button == 0:
                            self.START_KEY = True
                            self.debouncing_timer = 0
                        if event.button == 4:
                            self.BACK_KEY = True
                            self.debouncing_timer = 0
                        if event.button == self.game.COIN_KEY:
                            self.game.wallet.increase_balance()
                        if event.button == 2:
                            self.HIGHSCORE_KEY = True
                    if event.type == pygame.JOYAXISMOTION:
                        if event.axis == 1 and event.value > .95:
                            self.DOWN_KEY = True
                            self.debouncing_timer = 0
                        if event.axis == 1 and event.value < -.95:
                            self.UP_KEY = True
                            self.debouncing_timer = 0

    #move cursor depending on button press and start level on select
    def move_cursor(self):
        if self.START_KEY:
            if "Level" in self.state:
                if self.game.wallet.balance > 0:
                    self.red_wallet = False
                    mixer.music.unload()
                    self.level_select_sound.play()
                    if self.state == 'Level 1':
                        vid = Video("./assets/Intro/praktikumshortcutintro.mp4")
                        vid.set_size((WIDTH, HEIGHT))
                        # this is a loop that runs while the video is playing. We simply blit the current video frame
                        # onto the display surface
                        while vid.active:
                            for event in pygame.event.get():
                                if event.type == QUIT:
                                    pygame.quit()
                                    sys.exit()
                            vid.draw(displaysurface, (0,0))
                            FramePerSec.tick(60)
                            pygame.display.flip()
                        vid.close()
                        self.game.select_level(self.levels[0])
                    elif self.state == 'Level 2':
                        self.game.select_level(self.levels[1])
                    elif self.state == 'Level 3':
                        self.game.select_level(self.levels[2])
                    elif self.state == 'Level 4':
                        self.game.select_level(self.levels[3])
                    elif self.state == 'Level 5':
                        self.game.select_level(self.levels[4])
                    elif self.state == 'Level 6':
                        self.game.select_level(self.levels[5])
                    elif self.state == 'Level 7':
                        self.game.select_level(self.levels[6])
                    self.run_menu = False
                    self.game.curr_menu = self.game.main_menu
                    self.game.playing = True
                    self.game.inMenu = False
                    mixer.music.load('assets/music/dream_fire_without_beginning_silence.mp3')
                    mixer.music.set_volume(1.0)
                    mixer.music.play(loops=-1)
                else:
                    self.red_wallet = True
                    self.wallet_error_sound.play()

            if self.state == 'Back':
                self.enter_sound.play()
                self.run_menu = False
                self.game.curr_menu = self.game.main_menu
                self.game.curr_menu.run_menu = True

        if self.DOWN_KEY:
            self.select_sound.play()
            if self.state == 'Level 1':
                self.state = 'Level 2'
                self.level_index = 1
                self.cursor_pos = (self.level_title_coor[1][0] + self.cursor_offset_x, self.level_title_coor[1][1])
            elif self.state == 'Level 2':
                self.state = 'Level 3'
                self.level_index = 2
                self.cursor_pos = (self.level_title_coor[2][0] + self.cursor_offset_x, self.level_title_coor[2][1])
            elif self.state == 'Level 3':
                self.state = 'Level 4'
                self.level_index = 3
                self.cursor_pos = (self.level_title_coor[3][0] + self.cursor_offset_x, self.level_title_coor[3][1])
            elif self.state == 'Level 4':
                self.state = 'Level 5'
                self.level_index = 4
                self.cursor_pos = (self.level_title_coor[4][0] + self.cursor_offset_x, self.level_title_coor[4][1])
            elif self.state == 'Level 5':
                self.state = 'Level 6'
                self.level_index = 5
                self.cursor_pos = (self.level_title_coor[5][0] + self.cursor_offset_x, self.level_title_coor[5][1])
            elif self.state == 'Level 6':
                self.state = 'Level 7'
                self.level_index = 6
                self.cursor_pos = (self.level_title_coor[6][0] + self.cursor_offset_x, self.level_title_coor[6][1])
            elif self.state == 'Level 7':
                self.state = 'Back'
                self.cursor_pos = (self.back_x + self.cursor_offset_x, self.back_y)
            elif self.state == 'Back':
                self.state = 'Level 1'
                self.level_index = 0
                self.cursor_pos = (self.level_title_coor[0][0] + self.cursor_offset_x, self.level_title_coor[0][1])

        elif self.UP_KEY:
            self.select_sound.play()
            if self.state == 'Level 1':
                self.state = 'Back'
                self.cursor_pos = (self.back_x + self.cursor_offset_x, self.back_y)
            elif self.state == 'Level 2':
                self.state = 'Level 1'
                self.level_index = 0
                self.cursor_pos = (self.level_title_coor[0][0] + self.cursor_offset_x, self.level_title_coor[0][1])
            elif self.state == 'Level 3':
                self.state = 'Level 2'
                self.level_index = 1
                self.cursor_pos = (self.level_title_coor[1][0] + self.cursor_offset_x, self.level_title_coor[1][1])
            elif self.state == 'Level 4':
                self.state = 'Level 3'
                self.level_index = 2
                self.cursor_pos = (self.level_title_coor[2][0] + self.cursor_offset_x, self.level_title_coor[2][1])
            elif self.state == 'Level 5':
                self.state = 'Level 4'
                self.level_index = 3
                self.cursor_pos = (self.level_title_coor[3][0] + self.cursor_offset_x, self.level_title_coor[3][1])
            elif self.state == 'Level 6':
                self.state = 'Level 5'
                self.level_index = 4
                self.cursor_pos = (self.level_title_coor[4][0] + self.cursor_offset_x, self.level_title_coor[4][1])
            elif self.state == 'Level 7':
                self.state = 'Level 6'
                self.level_index = 5
                self.cursor_pos = (self.level_title_coor[5][0] + self.cursor_offset_x, self.level_title_coor[5][1])
            elif self.state == 'Back':
                self.state = 'Level 7'
                self.level_index = 6
                self.cursor_pos = (self.level_title_coor[6][0] + self.cursor_offset_x, self.level_title_coor[6][1])


    #if the highscore key is pressed, set self.show_database to show the pop up window cotaining the top 10
    def react_to_highscore(self):

        if self.HIGHSCORE_KEY:
            if self.counter_button_highscore % 2 == 0:
                self.counter_button_highscore += 1
                if "Level" in self.state:
                    #if the cursor is on a level and the highscore key is pressed, indicate that the pop up window should be displayed
                    self.show_database = True
                    self.cursor_active = False
                    self.enter_sound.play()
            else:
                self.select_sound.play()
                self.show_database = False
                self.cursor_active = True
                self.counter_button_highscore += 1


    #refactor text object into an entry object
    def as_entry(self, dct):
        return Entry(dct['name'], dct['time'], dct['coins'], dct['bone'])

    #check the amount of coins in the global wallet
    def check_wallet(self):
        if self.game.wallet.balance > 0:
            self.red_wallet = False
        else:
            self.red_wallet = True

    #display the level select menu
    def display_menu(self):
        #basic initialization before showing the levle select menu
        self.run_menu = True

        self.game.fire_channel.stop()

        #frame surrounding a thumbnail
        thumb_frame = pygame.image.load("assets/hud/thumbnail_frame.png")
        thumb_frame = pygame.transform.scale(thumb_frame, (147,193.5))

        #thumbnail for back state
        back_thumb = pygame.image.load("assets/logo/back_logo.png")
        back_thumb = pygame.transform.scale(back_thumb, (129, 175.5))

        #layered background
        bg1 = pygame.image.load("assets/background/plx-1.png")
        bg1 = pygame.transform.scale(bg1, (WIDTH + 300, HEIGHT))
        bg2 = pygame.image.load("assets/background/plx-2.png")
        bg2 = pygame.transform.scale(bg2, (WIDTH + 300, HEIGHT))
        bg3 = pygame.image.load("assets/background/plx-3.png")
        bg3 = pygame.transform.scale(bg3, (WIDTH + 300, HEIGHT))
        bg4 = pygame.image.load("assets/background/plx-4.png")
        bg4 = pygame.transform.scale(bg4, (WIDTH + 300, HEIGHT))
        bg5 = pygame.image.load("assets/background/plx-5.png")
        bg5 = pygame.transform.scale(bg5, (WIDTH + 300, HEIGHT))

        #opacity effect for background
        shadow = pygame.Surface((WIDTH, HEIGHT))  # the size of your rect
        shadow.set_alpha(90)  # alpha level
        shadow.fill((0, 0, 0))  # this fills the entire surface

        #lower bar to show controls
        blue_button = pygame.image.load("assets/hud/blue_button.PNG")
        #blue_button = pygame.transform.scale(blue_button, (25, 25))
        control_surf = create_text_surf("Show Highscores", 15, (255, 255, 255))

        #blitting components into one background surface
        lvl_bg = pygame.image.load("assets/hud/level_sel_bg1.png")
        background_surface = pygame.Surface((WIDTH, HEIGHT))
        background_surface.blit(bg1, (0, 0))
        background_surface.blit(bg2, (0, 0))
        background_surface.blit(bg3, (0, 0))
        background_surface.blit(bg4, (0, 0))
        background_surface.blit(bg5, (0, 0))
        background_surface.blit(shadow, (0, 0))
        background_surface.blit(lvl_bg, dest=(0, 0))
        background_surface.blit(thumb_frame, (550, 35))

        #components for difficulty scale
        diff_title = create_text_surf("Difficulty:", 15, (237, 189, 78))
        difficulty_bg = pygame.image.load("assets/hud/background_2.png")
        difficulty_bg = pygame.transform.scale(difficulty_bg, (215, difficulty_bg.get_height()))

        green = pygame.Surface((55, difficulty_bg.get_height() - 8))
        green.fill((77, 168, 82))
        easy_surf = create_text_surf("Easy", 15, (255, 255, 255))

        blue = pygame.Surface((65, difficulty_bg.get_height() - 8))
        blue.fill((45, 54, 168))
        medium_surf = create_text_surf("Medium", 15, (255, 255, 255))

        red = pygame.Surface((55, difficulty_bg.get_height() - 8))
        red.fill((163, 46, 38))
        hard_surf = create_text_surf("Hard", 15, (255, 255, 255))

        #draw level title and back title
        title_surf = create_text_surf("Choose a level!", 20, (237, 189, 78))
        background_surface.blit(title_surf, dest=(self.title_x, self.title_y))
        back_title_surf = create_text_surf("Back", 20, (237, 189, 78))
        background_surface.blit(back_title_surf, dest=(self.back_x, self.back_y))

        #draw level titles onto background
        for title in self.level_title_surfs:
            background_surface.blit(title, self.level_title_coor[self.level_title_surfs.index(title)])


        #database pop-up window
        #get data every time the menu gets displayed for the first time
        data_highscores = [None, None, None, None, None, None, None]

        #parse objects from .json files into entries in data_highscores array
        for i in range(0, 7):
            with open(self.level_highscores[i], 'r', encoding="utf-8-sig") as f:
                try:
                    data_highscores[i] = json.load(f, object_hook = self.as_entry)
                except json.decoder.JSONDecodeError:
                    pass

        self.data_as_text = []

        #parse entries into text and put them in global self.data_as_text array
        for array_of_entries in data_highscores:
            if array_of_entries != []:
                array_of_entries_to_text = []
                for entry in array_of_entries:
                    entry_to_text = []
                    name = create_text_surf(str(entry.name), 12,  (237, 189, 78))
                    entry_to_text.append(name)
                    bone = create_text_surf(str(entry.bone), 12,  (237, 189, 78))
                    entry_to_text.append(bone)
                    coins = create_text_surf(str(entry.coins), 12,  (237, 189, 78))
                    entry_to_text.append(coins)
                    time = create_text_surf(str(entry.time), 12,  (237, 189, 78))
                    entry_to_text.append(time)
                    array_of_entries_to_text.append(entry_to_text)
                self.data_as_text.append(array_of_entries_to_text)
            else:
                self.data_as_text.append(None)

        #background and text components of wallet
        coin_background = pygame.image.load("assets/hud/level_background.png")
        coin_background = pygame.transform.scale(coin_background, (215, coin_background.get_height()))
        level_cost_surf = create_text_surf("Level Cost:", 15, (237, 189, 78))
        cost_text_surf = create_text_surf("1", 15, (237, 189, 78))

        #display the menu
        while self.run_menu:
            #check for events and wallet state (changes state of wallet depnding on amount)
            self.check_events()
            self.check_wallet()

            #if the cursor is allowed to move, move it
            if self.cursor_active:
                self.move_cursor()

            #check whether or not the player wants to see the highscore
            self.react_to_highscore()

            #blit basic background surface
            displaysurface.blit(background_surface, dest=(0,0))

            #depending on level, blit level information like thumbnail, difficulty and prize
            if "Level" in self.state:
                displaysurface.blit(self.level_thumbnails[self.level_index], dest=(560, 45))
                displaysurface.blit(difficulty_bg, (517, 235))
                displaysurface.blit(diff_title, dest=((540, 240)))

                displaysurface.blit(blue_button, dest=(535, 358))
                displaysurface.blit(control_surf, dest=(565, 358))

                if "Easy" in self.level_difficulties[self.level_index]:
                    displaysurface.blit(green, dest=(660, 239))
                    displaysurface.blit(easy_surf, dest=(667, 240))

                if "Medium" in self.level_difficulties[self.level_index]:
                    displaysurface.blit(blue, dest=(650, 239))
                    displaysurface.blit(medium_surf, dest=(653, 240))

                if "Hard" in self.level_difficulties[self.level_index]:
                    displaysurface.blit(red, dest=(660, 239))
                    displaysurface.blit(hard_surf, dest=(667, 240))

            #if cursor is over back state, dont blit level information
            elif self.state == "Back":
                displaysurface.blit(back_thumb, dest=(560, 45))

            #blit the cursor
            displaysurface.blit(self.cursor_surf, dest=self.cursor_pos)

            #blit wallet information
            displaysurface.blit(coin_background, dest=(517, 270))
            displaysurface.blit(level_cost_surf, dest=(542, 310))
            displaysurface.blit(cost_text_surf, dest=(663, 310))

            #depedning on state of wallet found out earlier in the loop, display different color of wallet
            if self.red_wallet == True:
                displaysurface.blit(self.red_wallet_surf, dest=(542, 280))
            else:
                displaysurface.blit(self.yellow_wallet_surf, dest=(542, 280))
            displaysurface.blit(self.game.wallet.surf, dest=(663,280))

            #if the player wants to see the database, blit pop up window at center of the screen, depending on level index
            if self.show_database:
                displaysurface.blit(self.pop_up_window, dest=(WIDTH/2 - self.pop_up_window.get_width()/2,HEIGHT/2-self.pop_up_window.get_height()/2))
                displaysurface.blit(self.pop_up_titles[self.level_index],
                                   dest=(WIDTH / 2 - self.pop_up_titles[self.level_index].get_width()/2, 55))
                #blit the top10 entries
                self.blit_top10(self.data_as_text[self.level_index])

            #update the screen (method of menu class)
            self.blit_screen()

    #blit the top 10 entries in a loop
    def blit_top10(self, top10entries):
        if top10entries is not None:
            dest_y = 130
            for entry in top10entries:
                dest_x = 85
                for entry_elem in entry:
                    displaysurface.blit(entry_elem, dest=(dest_x,dest_y))
                    dest_x += 180
                dest_y += 20

        else:
            displaysurface.blit(self.no_highscores, dest=(WIDTH / 2 - self.no_highscores.get_width()/2, HEIGHT/2 - self.no_highscores.get_height()/2))