import time

import pygame
from pygame.locals import *

import pygame
import constants
import sprites
from constants import *
from sprites import obstacles, doors, ladders, dead, slideables
from spritesheet import Spritesheet
import pydash as _


class Player(pygame.sprite.Sprite):
    def __init__(self, player_1=False, player_2=False, start_position=(0,0), game_mode="Singleplayer", joystick=None):
        super().__init__()

        self.game_mode = game_mode

        # Movement Variables
        self.vel = vec(0, 0)
        self.acc = vec(0, 0)

        self.jumping = False
        self.lives = 3
        self.score = 0
        # According Variables are true or False according to which player this is
        self.player_1 = player_1
        self.player_2 = player_2
        self.start_position = start_position
        self.pos = start_position
        self.bone_score = 0

        self.p1_death_sound = pygame.mixer.Sound("assets/sounds/p1_death.wav")
        self.p2_death_sound = pygame.mixer.Sound("assets/sounds/dog_death.wav")
        # used for respawning
        self.last_obstacle_hits = None
        self.invincible_counter = 0

        # used for animations
        self.fps_counter = 0
        self.cur_animation_index = 0

        self.joystick = joystick

        self.jump_sound = pygame.mixer.Sound("assets/sounds/jump.wav")
        self.jump_sound.set_volume(0.4)

        # set correct graphics, sizes, sounds, and controls for each player
        if player_1:
            self.surf = pygame.Surface((32, 32))
            self.empty_surf = pygame.Surface(self.surf.get_size(), pygame.SRCALPHA, 32).convert_alpha()
            self.rect = pygame.Rect(0, 0, 12, 25)
            self.running_sound = pygame.mixer.Sound("assets/sounds/running_p1.wav")
            self.running_channel = pygame.mixer.Channel(4)
            self.running_sound.set_volume(0.2)
            self.MOVE_LEFT = K_LEFT
            self.MOVE_RIGHT = K_RIGHT
            self.MOVE_UP = K_UP
            self.MOVE_DOWN = K_DOWN
            self.BOX_PUSH = K_m
            self.pushing = False
            self.active_player = True

            spritesheet_idle = Spritesheet("./assets/player_1/Idle.png")
            self.idle_images = spritesheet_idle.images_at(
                [(32 * i, 0, 32, 32) for i in range(6)],
                colorkey=(0, 0, 0))

            spritesheet_run = Spritesheet("./assets/player_1/Run.png")
            self.run_images = spritesheet_run.images_at(
                [(32 * i, 0, 32, 32) for i in range(6)],
                colorkey=(0, 0, 0))

            spritesheet_fly = Spritesheet("./assets/player_1/Flying.png")
            self.fly_images = spritesheet_fly.images_at(
                [(32 * i, 0, 32, 32) for i in range(2)],
                colorkey=(0, 0, 0))

            spritesheet_jump = Spritesheet("./assets/player_1/Jump.png")
            self.jump_images = spritesheet_jump.images_at(
                [(32 * i, 0, 32, 32) for i in range(3)],
                colorkey=(0, 0, 0))

        elif player_2:
            self.crouching = False
            self.surf = pygame.Surface((32, 24))
            self.empty_surf = pygame.Surface(self.surf.get_size(), pygame.SRCALPHA, 32).convert_alpha()
            self.rect = pygame.Rect(0, 0, 16, 20)
            self.running_sound = pygame.mixer.Sound("assets/sounds/running_p2.wav")
            self.running_sound.set_volume(0.2)
            self.running_channel = pygame.mixer.Channel(5)
            if self.game_mode == "Multiplayer":
                self.MOVE_LEFT = K_a
                self.MOVE_RIGHT = K_d
                self.MOVE_UP = K_w
                self.MOVE_DOWN = K_s
                self.CROUCH = K_m
                self.active_player = True
            elif self.game_mode == "Singleplayer":
                self.MOVE_LEFT = K_LEFT
                self.MOVE_RIGHT = K_RIGHT
                self.MOVE_UP = K_UP
                self.MOVE_DOWN = K_DOWN
                self.CROUCH = K_m
                self.active_player = False

            spritesheet = Spritesheet("./assets/player_2/IndianaBonesDogBrown.png")
            self.idle_images = spritesheet.images_at(
                [(64 * i, 0, 64, 48) for i in range(11)], colorkey=(122, 97, 86))
            self.idle_images = [pygame.transform.smoothscale(image.convert_alpha(), (32, 24)) for image in self.idle_images]

            self.run_images = spritesheet.images_at(
                [(64 * i, 48, 64, 48) for i in range(8)], colorkey=(122, 97, 86))
            self.run_images = [pygame.transform.smoothscale(image.convert_alpha(), (32, 24)) for image in self.run_images]

            self.crouch_images = spritesheet.images_at(
                [(64 * i, 192, 64, 48) for i in range(5)], colorkey=(122, 97, 86))
            self.crouch_images = [pygame.transform.smoothscale(image.convert_alpha(), (32, 24)) for image in self.crouch_images]


    def move(self):
        # this y acceleration is gravity
        self.acc = vec(0, 0.4)

        pressed_keys = pygame.key.get_pressed()

        # we only move the active player in singleplayer
        if self.active_player or not self.game_mode == "Singleplayer":
            # ACC is a constant defined in constants.py
            if pressed_keys[self.MOVE_LEFT]:
                self.acc.x = -ACC
            if pressed_keys[self.MOVE_RIGHT]:
                self.acc.x = ACC
            if self.player_2 and self.crouching:
                # if P2 is crouching, we slow him down by a factor of .5
                self.acc.x = .2 * self.acc.x
            if self.player_1:
                # if player is pushing, he is able to move boxes
                self.pushing = pressed_keys[self.BOX_PUSH]

        # actual movement, FRIC is a constant defined in constants.py
        self.acc.x += self.vel.x * FRIC
        self.vel += self.acc
        self.pos += self.vel + 0.5 * self.acc

        # if we are moving in the x direction and the running sound is not playing yet, we play it
        if abs(self.vel.x) > 0 and not self.running_channel.get_busy():
            self.running_channel.play(self.running_sound, -1)

        # if we are not moving anymore, we stop the running sound
        if abs(self.vel.x) < .26:
            self.running_channel.stop()

        # this prevents the player from falling out of the screen (although this is not needed anymore as the levels
        # have borders anyways)
        if self.pos.x > WIDTH - 15:
            self.pos.x = WIDTH - 15
        if self.pos.x < 15:
            self.pos.x = 15

        # final adjustment of the player rect to the new calculated position
        self.rect.midbottom = self.pos

    def move_with_joystick(self):
        # basically move() with joystick controls
        # refer to move() for documentation
        self.acc = vec(0, 0.4)

        if not self.game_mode == "Singleplayer" or self.active_player:
            left_right_axis = self.joystick.get_axis(0)
            if left_right_axis < -.95:
                self.acc.x = -ACC
            if left_right_axis > .95:
                self.acc.x = ACC
            if self.player_2 and self.crouching:
                self.acc.x = .2 * self.acc.x
            if self.player_1:
                self.pushing = self.joystick.get_button(1) == 1  # Pushing is on RED Button

        self.acc.x += self.vel.x * FRIC
        self.vel += self.acc
        self.pos += self.vel + 0.5 * self.acc

        if abs(self.vel.x) > 0 and not self.running_channel.get_busy():
            self.running_channel.play(self.running_sound, -1)

        if abs(self.vel.x) < .26:
            self.running_channel.stop()

        if self.pos.x > WIDTH - 15:
            self.pos.x = WIDTH - 15
        if self.pos.x < 15:
            self.pos.x = 15

        self.rect.midbottom = self.pos

    def jump(self):
        hits = pygame.sprite.spritecollide(self, obstacles, False)
        # if we stand on ground and are not jumping yet, then we jump by setting the y velocity to -6
        if hits and not self.jumping:
            self.jump_sound.play()
            self.jumping = True
            self.vel.y = -6

    def cancel_jump(self):
        # this is triggered on the KEY_UP Event of the JUMP Button
        # we cancel the jump by reducing the Y vel to -3
        self.jump_sound.stop()
        if self.jumping:
            if self.vel.y < -3:
                self.vel.y = -3

    def update(self):
        # animation system, based on the players movement we use the correct surfaces
        if self.player_1:
            if abs(self.vel.y > 1):
                relevant_animation = self.fly_images
            elif abs(self.vel.x) > 1:
                relevant_animation = self.run_images
            else:
                relevant_animation = self.idle_images
        else:
            if self.crouching:
                relevant_animation = self.crouch_images
            elif abs(self.vel.x) > 1:
                relevant_animation = self.run_images
            else:
                relevant_animation = self.idle_images

        self.cur_animation_index = self.cur_animation_index % len(relevant_animation)

        self.fps_counter += 1
        if self.invincible_counter > 0:
            self.invincible_counter -= 1

        # every 5 frames we update the animation frame of the current animation
        if self.fps_counter % 5 == 0:
            self.fps_counter = 0
            self.cur_animation_index = (self.cur_animation_index + 1) % len(relevant_animation)

        # if we are invincible, we are invisible for 10 frames and then visible again for 10 frames
        if self.invincible_counter > 0 and 0 < self.invincible_counter % 20 < 10:
            self.surf = self.empty_surf
        # flipping of the animation frames based on the direction the player moves into
        elif self.vel.x < 0 and self.player_1:
            self.surf = pygame.transform.flip(relevant_animation[self.cur_animation_index], flip_x=True, flip_y=False)
        elif self.vel.x > 0 and self.player_2:
            self.surf = pygame.transform.flip(relevant_animation[self.cur_animation_index], flip_x=True, flip_y=False)
        else:
            self.surf = relevant_animation[self.cur_animation_index]

        # collision detection
        obstacle_hits = pygame.sprite.spritecollide(self, obstacles, False)

        # filter out boxes that are pushed from collision detection for P1
        if self.player_1:
            obstacle_hits = _.filter_(obstacle_hits, lambda x: not (x in slideables and x.is_pushed and not x.is_wall_left and not x.is_wall_right))

        # filter out half tiles when player 2 is crouching
        if self.player_2 and self.crouching:
            obstacle_hits = _.filter_(obstacle_hits, lambda x: not sprites.halfTiles.has(x))

        ladder_hits = pygame.sprite.spritecollide(self, ladders, False)
        dead_hits = pygame.sprite.spritecollide(self, dead, False)

        # if we are invisible, we add collisions to dead tiles (otherwise one could fall to the floor in some cases)
        if self.invincible_counter > 0:
            obstacle_hits = obstacle_hits + _.filter_(dead_hits, lambda x: not sprites.fires.has(x))

        # if we have at least 2 obstacles (which is usually a floor) we can use these tiles for respawning
        if len(obstacle_hits) > 1:
            self.last_obstacle_hits = obstacle_hits

        # if all obstacles have the same left coordinate, we hit a wall
        is_wall = len(obstacle_hits) > 1 \
                  and all(platform.rect.left == obstacle_hits[0].rect.left for platform in obstacle_hits)
        # detection if we hit a ceiling
        is_ceiling = len(obstacle_hits) > 0 \
                  and any(abs(self.rect.left - platform.rect.left) <= self.rect.width * .7 and self.rect.top > platform.rect.top for platform in obstacle_hits)
        # detection if there is a ladder above (that we can climb up)
        is_ladder_above = any([(ladder.rect.top - 16) < self.rect.top for ladder in ladder_hits])

        if self.vel.y > 0 and not is_ceiling:
            # fix player y position to the floor (s.t. we do not fall through it)
            if obstacle_hits:
                largest_y_platform = obstacle_hits[0]
                for sprite in obstacle_hits:
                    if constants.DEBUG:
                        sprite.touched = True
                    if sprite.rect.top > largest_y_platform.rect.top:
                        largest_y_platform = sprite
                if self.pos.y < largest_y_platform.rect.bottom and not is_wall:
                    self.pos.y = largest_y_platform.rect.top + 1
                    self.vel.y = 0
                    self.jumping = False

        # Ladder Climbing for Player 1
        if ladder_hits and is_ladder_above and self.player_1 and self.active_player:
            if not self.joystick:
                pressed_keys = pygame.key.get_pressed()
                up_pressed = pressed_keys[self.MOVE_UP]
            else:
                up_pressed = self.joystick.get_axis(1) < -0.95
            if up_pressed:
                self.pos.y = self.pos.y - 2
                self.vel.y = 0
                self.jumping = False
        # Here we reached the top of the ladder
        elif ladder_hits and not is_ladder_above:
            if not self.joystick:
                pressed_keys = pygame.key.get_pressed()
                down_pressed = pressed_keys[self.MOVE_DOWN]
            else:
                down_pressed = self.joystick.get_axis(1) > 0.95
            # if player pressed down, he falls through the ladder
            if down_pressed and not obstacle_hits and not self.jumping:
                self.vel.y = 0.4
                self.pos.y = self.rect.bottom + 2
            # otherwise (except player is jumping) the player y is fixed to top ladder
            elif not self.jumping:
                self.vel.y = 0
                self.pos.y = ladder_hits[0].rect.top + 1

        # cancel the jump once we hit a ceiling
        if self.vel.y < 0 and obstacle_hits and is_ceiling:
            relevant_sprites = obstacle_hits
            most_top_platform = relevant_sprites[0]
            for platform in relevant_sprites:
                if platform.rect.top < most_top_platform.rect.bottom:
                    most_top_platform = platform
            # TODO: This constant factor of 0.7 is completely guessed through trying out, I have no Idea why this works
            if abs(self.rect.top - most_top_platform.rect.top) <= self.rect.height * .7:
                self.vel.y = 0.4

        # collision detection with walls

        # if we are moving right, and we hit a wall, we fix the players x to the most right obstacle in the wall
        if self.vel.x > 0 and obstacle_hits and not is_ceiling:
            relevant_sprites = obstacle_hits
            most_right_platform = relevant_sprites[0]
            for platform in relevant_sprites:
                if platform.rect.left > most_right_platform.rect.left or (platform.rect.left >= most_right_platform.rect.left and platform.rect.top < most_right_platform.rect.top):
                    most_right_platform = platform
            # TODO: This constant factor of 0.7 is completely guessed through trying out, I have no Idea why this works
            if self.rect.left < most_right_platform.rect.left and abs(
                    self.rect.top - most_right_platform.rect.top) <= self.rect.height * 0.7:
                # this is needed s.t. the correct animation frames are used (ideally this would be 0)
                self.vel.x = 0.00000000001
                self.pos.x = most_right_platform.rect.bottomleft[0] - (self.rect.width // 2 - 1)

        # vice versa above
        if self.vel.x < 0 and obstacle_hits and not is_ceiling:
            relevant_sprites = obstacle_hits
            most_left_platform = relevant_sprites[0]
            for platform in relevant_sprites:
                if platform.rect.left < most_left_platform.rect.left or (platform.rect.left <= most_left_platform.rect.left and platform.rect.top < most_left_platform.rect.top):
                    most_left_platform = platform
            if self.rect.left > most_left_platform.rect.left and abs(self.rect.top - most_left_platform.rect.top) <= self.rect.height * 0.7:
                # this is needed s.t. the correct animation frames are used (ideally this would be 0)
                self.vel.x = -0.00000000001
                self.pos.x = most_left_platform.rect.bottomright[0] + (self.rect.width // 2 - 1)

        # Killing
        if dead_hits and any([dead_sprite.killing for dead_sprite in dead_hits]) and self.invincible_counter == 0:
            if self.player_1:
                self.p1_death_sound.play()
            if self.player_2:
                self.p2_death_sound.play()

            time.sleep(.5)
            # respawning at the last obstacle hit. We choose the highest of the hit obstacles s.t. we do not
            # respwan in floors
            min_top = min([obstacle.rect.top for obstacle in self.last_obstacle_hits])
            for obstacle in self.last_obstacle_hits:
                if dead_hits[0].rect.bottom == obstacle.rect.top and dead_hits[0].rect.left - 3 == obstacle.rect.left:
                    continue
                if obstacle.rect.top != min_top:
                    continue
                else:
                    self.pos = obstacle.rect.midtop
            self.vel = vec(0, 0)
            self.acc = vec(0, 0)
            self.jumping = False
            # self.running_sound.stop()
            self.lives -= 1
            # we are now invincible for 120 frames (2 seconds)
            self.invincible_counter = 120
